package ca.canadapost.cpcdp.tracking;

import ca.canadapost.cpcdp.tracking.stub.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.tracking.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetTrackingDetails Canada Post service.
 * <p>
 * The GetTrackingDetails service  returns all tracking events recorded for a specified
 * parcel. (The parcel is identified using a PIN or DNC).
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetTrackingDetailsClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try {
            // Your username and password are imported from the following file: CPCWS_SOAP_Tracking_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");

            TrackingService service = new TrackingService();
            TrackingPortType port = service.getTrackingPort();


            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            //ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/vis/soap/track");
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/vis/soap/track");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //client.getInInterceptors().add(new LoggingInInterceptor());
            //client.getOutInterceptors().add(new LoggingOutInterceptor());

            //build request
            GetTrackingDetailRequest request = new GetTrackingDetailRequest();
            request.setLocale("FR");

            //PIN Request
            request.setPin("1371134583769923");

            //DNC Request
            //request.setDnc("315052413796541");

            //execute request
            GetTrackingDetailResponse response = port.getTrackingDetail(request);

            //retrieve values from response
            if(response.getMessages() == null){
                System.out.println("PIN Number: " + response.getTrackingDetail().getPin());
                System.out.println("Signature Exists: " + response.getTrackingDetail().getSignatureImageExists());
                System.out.println("Suppress Signature: " + response.getTrackingDetail().getSuppressSignature());
            } else {
                // Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
